@extends('layouts.admin')
@section('content')
<div class="alert alert-primary" role="alert">
    Recommended homepage slider resolution is <strong>1100x400 px</strong> (width: 1100px and height: 400px).
</div>
<div class="row">
    <div class="col-12">
        <div class="card table-card rounded-0">
            <div class="card-header" style="padding: 12px 15px">
                <div class="d-sm-flex align-items-center justify-content-between">
                    <h5 class="mb-3 mb-sm-0">Home Page Slider</h5>
                    <div class="d-flex gap-1">
                        <button type="button" class="btn btn-light-primary border-primary rounded py-2 update-image" target-id="" target-type="add-slider"><i class="fas fa-plus"></i></button>
                    </div>
                </div>
            </div>
            <div class="card-body pb-0">
                <div class="table-responsive ">
                    <table id="data-table" class="display px-5">
                        <thead>
                            <tr>
                            </tr>
                        </thead>
                    </table>
                </div>
            </div>
        </div>
    </div>
</div>
<div
    class="offcanvas offcanvas-end"
    data-bs-scroll="true"
    tabindex="-1"
    id="slider-details"
    aria-labelledby="offcanvasWithBothOptionsLabel"
    >
    <div class="offcanvas-header border-bottom bg-primary">
        <h5 class="offcanvas-title text-uppercase text-white" id="offcanvasWithBothOptionsLabel">Slider Information</h5>
        <button type="button" class="btn-close text-reset text-white" data-bs-dismiss="offcanvas" aria-label="Close" style="color: white;"></button>
    </div>
    <div class="offcanvas-body"></div>
    <div class="offcanvas-footer">
        <div class="d-flex">
            <button type="button" class="btn btn-primary rounded-0 w-100 sliderUpdateBtn" id="rejectButton">
                <i class="fas fa-pencil-alt"></i> Update
            </button>
            <button type="button" class="btn btn-danger rounded-0 w-100 sliderDelBtn" id="successButton">
                <i class="fas fa-trash-alt"></i> Delete
            </button>
        </div>
    </div>
</div>
{!!$mediaModal!!}
@endsection
@section('footer_script')
<script>
    $(document).ready(function() {
        $.ajaxSetup({
            headers: {
                'X-CSRF-TOKEN': $('meta[name="csrf-token"]').attr('content')
            }
        });
        // Data Table
        var table = $('#data-table').DataTable({
            processing: false,
            serverSide: true,
            lengthChange: false,
            info: false,
            footer: false,
            ajax: {
                url: '{{ route('fetch_slider') }}',
                type: 'GET',
                beforeSend: function() {
                    $('.top-loader').show();
                },
                complete: function() {
                    $('.top-loader').hide();
                },
                error: function(xhr, error, thrown) {
                    new Notify({
                        status: 'error',
                        title: 'ERROR',
                        text: 'An error occurred while fetching data. Please try again',
                        effect: 'fade',
                        speed: 300,
                        customClass: '',
                        customIcon: '',
                        showIcon: true,
                        showCloseButton: false,
                        autoclose: true,
                        autotimeout: 3000,
                        notificationsGap: null,
                        notificationsPadding: null,
                        type: 'filled',
                        position: 'left bottom',
                        customWrapper: '',
                    });
                }
            },
            columns: [
                {
                    data: 'img',
                    title: 'IMAGE',
                    className: 'text-left',
                    orderable: false,
                    searchable: true,
                    render: function(data, type, row) {
                        return data ? `<img src="${data}" alt="Thumbnail" width="100" height="50" class="rounded" >` : 'No Image';
                    }
                },
                {
                    data: 'width',
                    title: 'RESOLUTION',
                    className: 'text-left',
                    orderable: true,
                    searchable: true,
                    render: function(data, type, row) {
                        if (data && row.height) {
                            return data + 'x' + row.height + ' px';
                        } else {
                            return '-';
                        }
                    }
                },
                {
                    data: 'id',
                    title: 'ACTION',
                    className: 'text-end',
                    orderable: true,
                    searchable: true,
                    render: function(data, type, row) {
                        return `
                            <button type="button"
                                class="btn btn-light-danger rounded py-1 px-2 delete-btn"
                                data-id="${data}">
                                <i class="fas fa-trash-alt"></i> Delete
                            </button>
                        `;
                    }
                },
            ],
            pageLength: -1,
            drawCallback: function(settings) {
                var api = this.api();
                var totalEntries = api.page.info().recordsTotal;
                $('.dataTables_wrapper .dataTables_paginate').hide();
            }
        });
        // Function to reload Service DataTable Globally
        window.refreshMainTable = function () {
            table.ajax.reload(null, false);
        };
        $(document).on('click', '.delete-btn', function(){
            var sliderID = $(this).data('id');
            if (confirm("Are you sure to delete this slider?")) {
                $.ajax({
                    url: '/admin/del-slider/' + sliderID,
                    method: 'GET',
                    beforeSend: function() {
                        $('.top-loader').show();
                    },
                    complete: function() {
                        $('.top-loader').hide();
                    },
                    success: function(response) {
                        if (response.success) {
                            $('#slider-details').offcanvas('hide');
                            notifier.show('Success', response.message, 'success', '/resource/ok-48.png', 10000);
                            refreshMainTable();
                        }
                        else{
                            notifier.show('ERROR',  response.message, 'danger', '/resource/high_priority-48.png', 10000);
                        }
                    },
                    error: function(xhr, status, error) {
                        notifier.show('ERROR',  'Error loading details: ' + error, 'danger', '/resource/high_priority-48.png', 10000);
                    },
                });
            }
        })
    });
</script>
{!!$mediaJs!!}
@endsection
